use std::collections::HashSet;
use common::{Day, Part};
use phf::phf_map;

static VALUES: phf::Map<char, usize> = phf_map!{
    'a' => 1,
    'b' => 2,
    'c' => 3,
    'd' => 4,
    'e' => 5,
    'f' => 6,
    'g' => 7,
    'h' => 8,
    'i' => 9,
    'j' => 10,
    'k' => 11,
    'l' => 12,
    'm' => 13,
    'n' => 14,
    'o' => 15,
    'p' => 16,
    'q' => 17,
    'r' => 18,
    's' => 19,
    't' => 20,
    'u' => 21,
    'v' => 22,
    'w' => 23,
    'x' => 24,
    'y' => 25,
    'z' => 26,
    'A' => 27,
    'B' => 28,
    'C' => 29,
    'D' => 30,
    'E' => 31,
    'F' => 32,
    'G' => 33,
    'H' => 34,
    'I' => 35,
    'J' => 36,
    'K' => 37,
    'L' => 38,
    'M' => 39,
    'N' => 40,
    'O' => 41,
    'P' => 42,
    'Q' => 43,
    'R' => 44,
    'S' => 45,
    'T' => 46,
    'U' => 47,
    'V' => 48,
    'W' => 49,
    'X' => 50,
    'Y' => 51,
    'Z' => 52,
};

pub fn run_day_and_part(day: Day, part: Part, inp: &str) -> usize {
    match (day, part) {
        (Day::Three, Part::One) => part01_parse(inp).iter().sum(),
        (Day::Three, Part::Two) => part02_parse(inp).iter().sum(),
        (_, _) => unimplemented!(),
    }
}

fn part01_parse(inp: &str) -> Vec<usize> {
    let inp = inp.split_whitespace();
    inp.map(|row: &str| row.split_at(row.len() / 2))
        .map(|(left, right): (&str, &str)| (HashSet::from_iter(left.chars()),
                              HashSet::from_iter(right.chars())))
        .map(|(left, right): (HashSet<char>, HashSet<char>)| *left.intersection(&right).next().unwrap())
        .map(|i: char| VALUES[&i])
        .collect::<Vec<usize>>()
}

fn part02_parse(inp: &str) -> Vec<usize> {
    vec![]
}


#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn part_01_example_works() {
        let input = r#"vJrwpWtwJgWrhcsFMMfFFhFp
jqHRNqRjqzjGDLGLrsFMfFZSrLrFZsSL
PmmdzqPrVvPwwTWBwg
wMqvLMZHhHMvwLHjbvcjnnSBnvTQFn
ttgJtRGJQctTZtZT
CrZsJsPPZsGzwwsLwLmpwMDw"#;
        let result = 2 + 2 ;
        assert_eq!(result, 4);
    }
}
